@@ -63,8 +63,13 @@ GST_STATIC_PAD_TEMPLATE ("src",
     );
 
 #define gst_tone_generate_src_parent_class parent_class
-G_DEFINE_TYPE (GstToneGenerateSrc, gst_tone_generate_src, GST_TYPE_PUSH_SRC);
+G_DEFINE_TYPE_WITH_CODE (GstToneGenerateSrc, gst_tone_generate_src,
+    GST_TYPE_PUSH_SRC, GST_DEBUG_CATEGORY_INIT (tone_generate_src_debug,
+        "tonegeneratesrc", 0, "Telephony Tone Test Source");
+    );
 
+GST_ELEMENT_REGISTER_DEFINE (tonegeneratesrc, "tonegeneratesrc", GST_RANK_NONE,
+    GST_TYPE_TONE_GENERATE_SRC);
 static void gst_tone_generate_src_finalize (GObject * object);
 static void gst_tone_generate_src_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec);
@@ -403,13 +408,3 @@ gst_tone_generate_src_get_property (GObject * object, guint prop_id,
       break;
   }
 }
-
-gboolean
-gst_tone_generate_src_plugin_init (GstPlugin * plugin)
-{
-  GST_DEBUG_CATEGORY_INIT (tone_generate_src_debug, "tonegeneratesrc", 0,
-      "Telephony Tone Test Source");
-
-  return gst_element_register (plugin, "tonegeneratesrc",
-      GST_RANK_NONE, GST_TYPE_TONE_GENERATE_SRC);
-}
