@@ -32,6 +32,8 @@
 
 #include "rtptransceiver.h"
 
+#include "webrtc-priv.h"
+
 #define GST_CAT_DEFAULT gst_webrtc_rtp_transceiver_debug
 GST_DEBUG_CATEGORY_STATIC (GST_CAT_DEFAULT);
 
@@ -51,11 +53,14 @@ enum
 enum
 {
   PROP_0,
-  PROP_MID,
   PROP_SENDER,
   PROP_RECEIVER,
   PROP_DIRECTION,
   PROP_MLINE,
+  PROP_MID,
+  PROP_CURRENT_DIRECTION,
+  PROP_KIND,
+  PROP_CODEC_PREFERENCES,
   PROP_STOPPED,                 // FIXME
 };
 
@@ -78,7 +83,14 @@ gst_webrtc_rtp_transceiver_set_property (GObject * object, guint prop_id,
       webrtc->mline = g_value_get_uint (value);
       break;
     case PROP_DIRECTION:
+      GST_OBJECT_LOCK (webrtc);
       webrtc->direction = g_value_get_enum (value);
+      GST_OBJECT_UNLOCK (webrtc);
+      break;
+    case PROP_CODEC_PREFERENCES:
+      GST_OBJECT_LOCK (webrtc);
+      gst_caps_replace (&webrtc->codec_preferences, g_value_get_boxed (value));
+      GST_OBJECT_UNLOCK (webrtc);
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -93,6 +105,9 @@ gst_webrtc_rtp_transceiver_get_property (GObject * object, guint prop_id,
   GstWebRTCRTPTransceiver *webrtc = GST_WEBRTC_RTP_TRANSCEIVER (object);
 
   switch (prop_id) {
+    case PROP_MID:
+      g_value_set_string (value, webrtc->mid);
+      break;
     case PROP_SENDER:
       g_value_set_object (value, webrtc->sender);
       break;
@@ -103,7 +118,20 @@ gst_webrtc_rtp_transceiver_get_property (GObject * object, guint prop_id,
       g_value_set_uint (value, webrtc->mline);
       break;
     case PROP_DIRECTION:
+      GST_OBJECT_LOCK (webrtc);
       g_value_set_enum (value, webrtc->direction);
+      GST_OBJECT_UNLOCK (webrtc);
+      break;
+    case PROP_CURRENT_DIRECTION:
+      g_value_set_enum (value, webrtc->current_direction);
+      break;
+    case PROP_KIND:
+      g_value_set_enum (value, webrtc->kind);
+      break;
+    case PROP_CODEC_PREFERENCES:
+      GST_OBJECT_LOCK (webrtc);
+      gst_value_set_caps (value, webrtc->codec_preferences);
+      GST_OBJECT_UNLOCK (webrtc);
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -199,6 +227,73 @@ gst_webrtc_rtp_transceiver_class_init (GstWebRTCRTPTransceiverClass * klass)
           GST_TYPE_WEBRTC_RTP_TRANSCEIVER_DIRECTION,
           GST_WEBRTC_RTP_TRANSCEIVER_DIRECTION_NONE,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+
+  /**
+   * GstWebRTCRTPTransceiver:mid:
+   *
+   * The media ID of the m-line associated with this transceiver. This
+   * association is established, when possible, whenever either a
+   * local or remote description is applied. This field is null if
+   * neither a local or remote description has been applied, or if its
+   * associated m-line is rejected by either a remote offer or any
+   * answer.
+   *
+   * Since: 1.20
+   */
+  g_object_class_install_property (gobject_class,
+      PROP_MID,
+      g_param_spec_string ("mid", "Media ID",
+          "The media ID of the m-line associated with this transceiver. This "
+          " association is established, when possible, whenever either a local"
+          " or remote description is applied. This field is null if neither a"
+          " local or remote description has been applied, or if its associated"
+          " m-line is rejected by either a remote offer or any answer.",
+          NULL, G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+
+  /**
+   * GstWebRTCRTPTransceiver:current-direction:
+   *
+   * The transceiver's current directionality, or none if the
+   * transceiver is stopped or has never participated in an exchange
+   * of offers and answers. To change the transceiver's
+   * directionality, set the value of the direction property.
+   *
+   * Since: 1.20
+   **/
+  g_object_class_install_property (gobject_class,
+      PROP_DIRECTION,
+      g_param_spec_enum ("current-direction", "Current Direction",
+          "Transceiver current direction",
+          GST_TYPE_WEBRTC_RTP_TRANSCEIVER_DIRECTION,
+          GST_WEBRTC_RTP_TRANSCEIVER_DIRECTION_NONE,
+          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+
+  /**
+   * GstWebRTCRTPTransceiver:kind:
+   *
+   * The kind of media this transceiver transports
+   *
+   * Since: 1.20
+   **/
+  g_object_class_install_property (gobject_class,
+      PROP_KIND,
+      g_param_spec_enum ("kind", "Media Kind",
+          "Kind of media this transceiver transports",
+          GST_TYPE_WEBRTC_KIND, GST_WEBRTC_KIND_UNKNOWN,
+          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+
+  /**
+   * GstWebRTCRTPTransceiver:codec-preferences:
+   *
+   * Caps representing the codec preferences.
+   *
+   * Since: 1.20
+   **/
+  g_object_class_install_property (gobject_class,
+      PROP_CODEC_PREFERENCES,
+      g_param_spec_boxed ("codec-preferences", "Codec Preferences",
+          "Caps representing the codec preferences.",
+          GST_TYPE_CAPS, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 }
 
 static void
