@@ -15,8 +15,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -30,14 +30,13 @@
 static gboolean
 plugin_init (GstPlugin * plugin)
 {
-  if (!gst_viewfinder_bin_plugin_init (plugin))
-    return FALSE;
-  if (!gst_wrapper_camera_bin_src_plugin_init (plugin))
-    return FALSE;
-  if (!gst_camera_bin2_plugin_init (plugin))
-    return FALSE;
+  gboolean ret = FALSE;
 
-  return TRUE;
+  ret |= GST_ELEMENT_REGISTER (viewfinderbin, plugin);
+  ret |= GST_ELEMENT_REGISTER (wrappercamerabinsrc, plugin);
+  ret |= GST_ELEMENT_REGISTER (camerabin, plugin);
+
+  return ret;
 }
 
 GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
