@@ -590,6 +590,11 @@ static const guint8 sony_mpeg4_extradata[] = {
 
 /* RP224 */
 
+static const MXFUL video_mpeg_compression = { {
+        0x06, 0x0E, 0x2B, 0x34, 0x04, 0x01, 0x01, 0x01, 0x04, 0x01, 0x02, 0x02,
+    0x01,}
+};
+
 static const MXFUL sound_essence_compression_ac3 = { {
         0x06, 0x0E, 0x2B, 0x34, 0x04, 0x01, 0x01, 0x01, 0x04, 0x02, 0x02, 0x02,
     0x03, 0x02, 0x01, 0x00}
@@ -627,20 +632,23 @@ static const MXFUL sound_essence_compression_aac = { {
 
 static GstCaps *
 mxf_mpeg_es_create_caps (MXFMetadataTimelineTrack * track, GstTagList ** tags,
-    MXFEssenceElementHandleFunc * handler, gpointer * mapping_data,
-    MXFMetadataGenericPictureEssenceDescriptor * p,
+    gboolean * intra_only, MXFEssenceElementHandleFunc * handler,
+    gpointer * mapping_data, MXFMetadataGenericPictureEssenceDescriptor * p,
     MXFMetadataGenericSoundEssenceDescriptor * s)
 {
   GstCaps *caps = NULL;
   const gchar *codec_name = NULL;
   MXFMPEGEssenceType t, *mdata;
+  gchar str[48];
 
   *mapping_data = g_malloc (sizeof (MXFMPEGEssenceType));
   mdata = (MXFMPEGEssenceType *) * mapping_data;
 
   /* SMPTE RP224 */
   if (p) {
-    if (mxf_ul_is_zero (&p->picture_essence_coding)) {
+    if (mxf_ul_is_zero (&p->picture_essence_coding) ||
+        mxf_ul_is_equal (&p->picture_essence_coding,
+            &p->parent.essence_container)) {
       GST_WARNING ("No picture essence coding defined, assuming MPEG2");
       caps =
           gst_caps_new_simple ("video/mpeg", "mpegversion", G_TYPE_INT, 2,
@@ -648,19 +656,11 @@ mxf_mpeg_es_create_caps (MXFMetadataTimelineTrack * track, GstTagList ** tags,
       codec_name = "MPEG-2 Video";
       t = MXF_MPEG_ESSENCE_TYPE_VIDEO_MPEG2;
       memcpy (mdata, &t, sizeof (MXFMPEGEssenceType));
-    } else if (p->picture_essence_coding.u[0] != 0x06
-        || p->picture_essence_coding.u[1] != 0x0e
-        || p->picture_essence_coding.u[2] != 0x2b
-        || p->picture_essence_coding.u[3] != 0x34
-        || p->picture_essence_coding.u[4] != 0x04
-        || p->picture_essence_coding.u[5] != 0x01
-        || p->picture_essence_coding.u[6] != 0x01
-        || p->picture_essence_coding.u[8] != 0x04
-        || p->picture_essence_coding.u[9] != 0x01
-        || p->picture_essence_coding.u[10] != 0x02
-        || p->picture_essence_coding.u[11] != 0x02
-        || p->picture_essence_coding.u[12] != 0x01) {
-      GST_ERROR ("No MPEG picture essence coding");
+      *intra_only = FALSE;
+    } else if (!mxf_ul_is_subclass (&video_mpeg_compression,
+            &p->picture_essence_coding)) {
+      GST_ERROR ("Not MPEG picture essence coding %s",
+          mxf_ul_to_string (&p->picture_essence_coding, str));
       caps = NULL;
     } else if (p->picture_essence_coding.u[13] >= 0x01 &&
         p->picture_essence_coding.u[13] <= 0x08) {
@@ -669,12 +669,14 @@ mxf_mpeg_es_create_caps (MXFMetadataTimelineTrack * track, GstTagList ** tags,
       codec_name = "MPEG-2 Video";
       t = MXF_MPEG_ESSENCE_TYPE_VIDEO_MPEG2;
       memcpy (mdata, &t, sizeof (MXFMPEGEssenceType));
+      *intra_only = FALSE;
     } else if (p->picture_essence_coding.u[13] == 0x10) {
       caps = gst_caps_new_simple ("video/mpeg", "mpegversion", G_TYPE_INT, 1,
           "systemstream", G_TYPE_BOOLEAN, FALSE, NULL);
       codec_name = "MPEG-1 Video";
       t = MXF_MPEG_ESSENCE_TYPE_VIDEO_MPEG2;
       memcpy (mdata, &t, sizeof (MXFMPEGEssenceType));
+      *intra_only = TRUE;
     } else if (p->picture_essence_coding.u[13] == 0x20) {
       MXFLocalTag *local_tag =
           (((MXFMetadataBase *) p)->other_tags) ?
@@ -698,6 +700,7 @@ mxf_mpeg_es_create_caps (MXFMetadataTimelineTrack * track, GstTagList ** tags,
       codec_name = "MPEG-4 Video";
       t = MXF_MPEG_ESSENCE_TYPE_VIDEO_MPEG4;
       memcpy (mdata, &t, sizeof (MXFMPEGEssenceType));
+      *intra_only = FALSE;
     } else if ((p->picture_essence_coding.u[13] >> 4) == 0x03) {
       /* RP 2008 */
 
@@ -707,6 +710,7 @@ mxf_mpeg_es_create_caps (MXFMetadataTimelineTrack * track, GstTagList ** tags,
       codec_name = "h.264 Video";
       t = MXF_MPEG_ESSENCE_TYPE_VIDEO_AVC;
       memcpy (mdata, &t, sizeof (MXFMPEGEssenceType));
+      *intra_only = FALSE;
     } else {
       GST_ERROR ("Unsupported MPEG picture essence coding 0x%02x",
           p->picture_essence_coding.u[13]);
@@ -764,6 +768,7 @@ mxf_mpeg_es_create_caps (MXFMetadataTimelineTrack * track, GstTagList ** tags,
       mxf_metadata_generic_sound_essence_descriptor_set_caps (s, caps);
       *handler = mxf_mpeg_audio_handle_essence_element;
     }
+    *intra_only = TRUE;
   }
 
   if (caps) {
@@ -823,7 +828,8 @@ mxf_mpeg_get_track_wrapping (const MXFMetadataTimelineTrack * track)
 
 static GstCaps *
 mxf_mpeg_create_caps (MXFMetadataTimelineTrack * track, GstTagList ** tags,
-    MXFEssenceElementHandleFunc * handler, gpointer * mapping_data)
+    gboolean * intra_only, MXFEssenceElementHandleFunc * handler,
+    gpointer * mapping_data)
 {
   MXFMetadataFileDescriptor *f = NULL;
   MXFMetadataGenericPictureEssenceDescriptor *p = NULL;
@@ -866,7 +872,9 @@ mxf_mpeg_create_caps (MXFMetadataTimelineTrack * track, GstTagList ** tags,
   if (f->essence_container.u[13] == 0x04) {
     GST_DEBUG ("Found MPEG ES stream");
 
-    caps = mxf_mpeg_es_create_caps (track, tags, handler, mapping_data, p, s);
+    caps =
+        mxf_mpeg_es_create_caps (track, tags, intra_only, handler, mapping_data,
+        p, s);
   } else if (f->essence_container.u[13] == 0x07) {
     GST_ERROR ("MPEG PES streams not supported yet");
     return NULL;
@@ -880,6 +888,7 @@ mxf_mpeg_create_caps (MXFMetadataTimelineTrack * track, GstTagList ** tags,
       *tags = gst_tag_list_new_empty ();
     gst_tag_list_add (*tags, GST_TAG_MERGE_APPEND, GST_TAG_VIDEO_CODEC,
         "MPEG PS", NULL);
+    *intra_only = FALSE;
   } else if (f->essence_container.u[13] == 0x09) {
     GST_DEBUG ("Found MPEG TS stream");
     caps = gst_caps_new_empty_simple ("video/mpegts");
@@ -888,6 +897,7 @@ mxf_mpeg_create_caps (MXFMetadataTimelineTrack * track, GstTagList ** tags,
       *tags = gst_tag_list_new_empty ();
     gst_tag_list_add (*tags, GST_TAG_MERGE_APPEND, GST_TAG_VIDEO_CODEC,
         "MPEG TS", NULL);
+    *intra_only = FALSE;
   } else if (f->essence_container.u[13] == 0x0f) {
     GST_DEBUG ("Found h264 NAL unit stream");
     /* RP 2008 */
@@ -899,6 +909,7 @@ mxf_mpeg_create_caps (MXFMetadataTimelineTrack * track, GstTagList ** tags,
       *tags = gst_tag_list_new_empty ();
     gst_tag_list_add (*tags, GST_TAG_MERGE_APPEND, GST_TAG_VIDEO_CODEC,
         "h.264 Video", NULL);
+    *intra_only = FALSE;
   } else if (f->essence_container.u[13] == 0x10) {
     GST_DEBUG ("Found h264 byte-stream stream");
     /* RP 2008 */
@@ -910,6 +921,7 @@ mxf_mpeg_create_caps (MXFMetadataTimelineTrack * track, GstTagList ** tags,
       *tags = gst_tag_list_new_empty ();
     gst_tag_list_add (*tags, GST_TAG_MERGE_APPEND, GST_TAG_VIDEO_CODEC,
         "h.264 Video", NULL);
+    *intra_only = FALSE;
   }
 
   if (p && caps)
@@ -1309,7 +1321,7 @@ mxf_mpeg_video_get_descriptor (GstPadTemplate * tmpl, GstCaps * caps,
         codec_data = gst_value_get_buffer (v);
         gst_buffer_map ((GstBuffer *) codec_data, &map, GST_MAP_READ);
         t->size = map.size;
-        t->data = g_memdup (map.data, map.size);
+        t->data = g_memdup2 (map.data, map.size);
         gst_buffer_unmap ((GstBuffer *) codec_data, &map);
         memcpy (&t->ul, &sony_mpeg4_extradata, 16);
         mxf_local_tag_insert (t, &MXF_METADATA_BASE (ret)->other_tags);
