@@ -38,6 +38,7 @@
 #include "config.h"
 #endif
 
+#include "gstrtmp2elements.h"
 #include "gstrtmp2sink.h"
 
 #include "gstrtmp2locationhandler.h"
@@ -67,6 +68,7 @@ typedef struct
   gboolean async_connect;
   guint peak_kbps;
   guint32 chunk_size;
+  GstRtmpStopCommands stop_commands;
   GstStructure *stats;
 
   /* If both self->lock and OBJECT_LOCK are needed,
@@ -106,6 +108,7 @@ static void gst_rtmp2_sink_uri_handler_init (GstURIHandlerInterface * iface);
 /* GstBaseSink virtual functions */
 static gboolean gst_rtmp2_sink_start (GstBaseSink * sink);
 static gboolean gst_rtmp2_sink_stop (GstBaseSink * sink);
+static gboolean gst_rtmp2_sink_event (GstBaseSink * sink, GstEvent * event);
 static gboolean gst_rtmp2_sink_unlock (GstBaseSink * sink);
 static gboolean gst_rtmp2_sink_unlock_stop (GstBaseSink * sink);
 static GstFlowReturn gst_rtmp2_sink_render (GstBaseSink * sink,
@@ -147,6 +150,7 @@ enum
   PROP_PEAK_KBPS,
   PROP_CHUNK_SIZE,
   PROP_STATS,
+  PROP_STOP_COMMANDS,
 };
 
 /* pad templates */
@@ -164,6 +168,8 @@ G_DEFINE_TYPE_WITH_CODE (GstRtmp2Sink, gst_rtmp2_sink, GST_TYPE_BASE_SINK,
     G_IMPLEMENT_INTERFACE (GST_TYPE_URI_HANDLER,
         gst_rtmp2_sink_uri_handler_init);
     G_IMPLEMENT_INTERFACE (GST_TYPE_RTMP_LOCATION_HANDLER, NULL));
+GST_ELEMENT_REGISTER_DEFINE_WITH_CODE (rtmp2sink, "rtmp2sink",
+    GST_RANK_PRIMARY + 1, GST_TYPE_RTMP2_SINK, rtmp2_element_init (plugin));
 
 static void
 gst_rtmp2_sink_class_init (GstRtmp2SinkClass * klass)
@@ -183,6 +189,7 @@ gst_rtmp2_sink_class_init (GstRtmp2SinkClass * klass)
   gobject_class->finalize = gst_rtmp2_sink_finalize;
   base_sink_class->start = GST_DEBUG_FUNCPTR (gst_rtmp2_sink_start);
   base_sink_class->stop = GST_DEBUG_FUNCPTR (gst_rtmp2_sink_stop);
+  base_sink_class->event = GST_DEBUG_FUNCPTR (gst_rtmp2_sink_event);
   base_sink_class->unlock = GST_DEBUG_FUNCPTR (gst_rtmp2_sink_unlock);
   base_sink_class->unlock_stop = GST_DEBUG_FUNCPTR (gst_rtmp2_sink_unlock_stop);
   base_sink_class->render = GST_DEBUG_FUNCPTR (gst_rtmp2_sink_render);
@@ -227,6 +234,19 @@ gst_rtmp2_sink_class_init (GstRtmp2SinkClass * klass)
       g_param_spec_boxed ("stats", "Stats", "Retrieve a statistics structure",
           GST_TYPE_STRUCTURE, G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
+  /**
+   * GstRtmp2Sink:stop-commands:
+   *
+   * Which commands (if any) to send on EOS event before closing connection
+   *
+   * Since: 1.20
+   */
+  g_object_class_install_property (gobject_class, PROP_STOP_COMMANDS,
+      g_param_spec_flags ("stop-commands", "Stop commands",
+          "RTMP commands to send on EOS event before closing connection",
+          GST_TYPE_RTMP_STOP_COMMANDS, GST_RTMP_DEFAULT_STOP_COMMANDS,
+          (GParamFlags) (G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS)));
+
   gst_type_mark_as_plugin_api (GST_TYPE_RTMP_LOCATION_HANDLER, 0);
   GST_DEBUG_CATEGORY_INIT (gst_rtmp2_sink_debug_category, "rtmp2sink", 0,
       "debug category for rtmp2sink element");
@@ -239,6 +259,7 @@ gst_rtmp2_sink_init (GstRtmp2Sink * self)
   self->location.publish = TRUE;
   self->async_connect = TRUE;
   self->chunk_size = GST_RTMP_DEFAULT_CHUNK_SIZE;
+  self->stop_commands = GST_RTMP_DEFAULT_STOP_COMMANDS;
 
   g_mutex_init (&self->lock);
   g_cond_init (&self->cond);
@@ -360,6 +381,11 @@ gst_rtmp2_sink_set_property (GObject * object, guint property_id,
       set_chunk_size (self);
       g_mutex_unlock (&self->lock);
       break;
+    case PROP_STOP_COMMANDS:
+      GST_OBJECT_LOCK (self);
+      self->stop_commands = g_value_get_flags (value);
+      GST_OBJECT_UNLOCK (self);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
       break;
@@ -457,6 +483,11 @@ gst_rtmp2_sink_get_property (GObject * object, guint property_id,
     case PROP_STATS:
       g_value_take_boxed (value, gst_rtmp2_sink_get_stats (self));
       break;
+    case PROP_STOP_COMMANDS:
+      GST_OBJECT_LOCK (self);
+      g_value_set_flags (value, self->stop_commands);
+      GST_OBJECT_UNLOCK (self);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
       break;
@@ -556,6 +587,47 @@ gst_rtmp2_sink_stop (GstBaseSink * sink)
   return TRUE;
 }
 
+static gboolean
+stop_publish_invoker (gpointer user_data)
+{
+  GstRtmp2Sink *self = user_data;
+
+  if (self->connection) {
+    GST_OBJECT_LOCK (self);
+    if (self->stop_commands != GST_RTMP_STOP_COMMANDS_NONE) {
+      gst_rtmp_client_stop_publish (self->connection, self->location.stream,
+          self->stop_commands);
+    }
+    GST_OBJECT_UNLOCK (self);
+  }
+
+  return G_SOURCE_REMOVE;
+}
+
+static gboolean
+gst_rtmp2_sink_event (GstBaseSink * sink, GstEvent * event)
+{
+  GstEventType type;
+  GstRtmp2Sink *self = GST_RTMP2_SINK (sink);
+
+  type = GST_EVENT_TYPE (event);
+
+  switch (type) {
+    case GST_EVENT_EOS:
+      g_mutex_lock (&self->lock);
+      if (self->loop) {
+        GST_DEBUG_OBJECT (self, "Got EOS: stopping publish");
+        g_main_context_invoke (self->context, stop_publish_invoker, self);
+      }
+      g_mutex_unlock (&self->lock);
+      break;
+    default:
+      break;
+  }
+
+  return GST_BASE_SINK_CLASS (gst_rtmp2_sink_parent_class)->event (sink, event);
+}
+
 static gboolean
 gst_rtmp2_sink_unlock (GstBaseSink * sink)
 {
@@ -1147,18 +1219,7 @@ gst_rtmp2_sink_get_stats (GstRtmp2Sink * self)
   g_mutex_lock (&self->lock);
 
   if (self->connection) {
-    GstRtmpConnection *connection = g_object_ref (self->connection);
-
-    g_mutex_unlock (&self->lock);
-
-    /* We need to do this without holding the lock as the g_async_queue_pop
-     * waits on the loop thread to deliver the stats. The loop thread might
-     * attempt to take the lock as well, leading to a deadlock. */
-    s = gst_rtmp_connection_get_stats (connection);
-
-    g_mutex_lock (&self->lock);
-
-    g_object_unref (connection);
+    s = gst_rtmp_connection_get_stats (self->connection);
   } else if (self->stats) {
     s = gst_structure_copy (self->stats);
   } else {
