@@ -263,7 +263,7 @@ static GstVideoDecoderClass *parent_class = NULL;
 GType
 gst_amc_video_dec_get_type (void)
 {
-  static volatile gsize type = 0;
+  static gsize type = 0;
 
   if (g_once_init_enter (&type)) {
     GType _type;
@@ -1811,7 +1811,7 @@ gst_amc_video_dec_set_format (GstVideoDecoder * decoder,
     GstMapInfo cminfo;
 
     gst_buffer_map (state->codec_data, &cminfo, GST_MAP_READ);
-    codec_data = g_memdup (cminfo.data, cminfo.size);
+    codec_data = g_memdup2 (cminfo.data, cminfo.size);
     codec_data_size = cminfo.size;
 
     is_format_change |= (!self->codec_data
