@@ -52,6 +52,7 @@ struct _GstAmcVideoEnc
   GstVideoEncoder parent;
 
   /* < private > */
+  GMutex codec_lock; // Protect creation / destruction of the codec
   GstAmcCodec *codec;
   GstAmcFormat *amc_format;
 
@@ -62,7 +63,7 @@ struct _GstAmcVideoEnc
   GstAmcColorFormatInfo color_format_info;
 
   guint bitrate;
-  guint i_frame_int;
+  gfloat i_frame_int;
 
   /* TRUE if the component is configured and saw
    * the first buffer */
