@@ -55,7 +55,6 @@
 #include <xf86drm.h>
 #include <xf86drmMode.h>
 #include <drm_fourcc.h>
-
 #include <string.h>
 
 #include "gstkmssink.h"
@@ -82,6 +81,8 @@ G_DEFINE_TYPE_WITH_CODE (GstKMSSink, gst_kms_sink, GST_TYPE_VIDEO_SINK,
     GST_DEBUG_CATEGORY_GET (CAT_PERFORMANCE, "GST_PERFORMANCE");
     G_IMPLEMENT_INTERFACE (GST_TYPE_VIDEO_OVERLAY,
         gst_kms_sink_video_overlay_init));
+GST_ELEMENT_REGISTER_DEFINE (kmssink, GST_PLUGIN_NAME, GST_RANK_SECONDARY,
+    GST_TYPE_KMS_SINK);
 
 enum
 {
@@ -176,7 +177,7 @@ kms_open (gchar ** driver)
 {
   static const char *drivers[] = { "i915", "radeon", "nouveau", "vmwgfx",
     "exynos", "amdgpu", "imx-drm", "rockchip", "atmel-hlcdc", "msm",
-    "xlnx", "vc4", "meson", "sun4i-drm", "mxsfb-drm",
+    "xlnx", "vc4", "meson", "sun4i-drm", "mxsfb-drm", "tegra",
     "xilinx_drm",               /* DEPRECATED. Replaced by xlnx */
   };
   int i, fd = -1;
@@ -416,6 +417,14 @@ get_drm_caps (GstKMSSink * self)
   return TRUE;
 }
 
+static void
+ensure_kms_allocator (GstKMSSink * self)
+{
+  if (self->allocator)
+    return;
+  self->allocator = gst_kms_allocator_new (self->fd);
+}
+
 static gboolean
 configure_mode_setting (GstKMSSink * self, GstVideoInfo * vinfo)
 {
@@ -437,6 +446,7 @@ configure_mode_setting (GstKMSSink * self, GstVideoInfo * vinfo)
 
   GST_INFO_OBJECT (self, "configuring mode setting");
 
+  ensure_kms_allocator (self);
   kmsmem = (GstKMSMemory *) gst_kms_allocator_bo_alloc (self->allocator, vinfo);
   if (!kmsmem)
     goto bo_failed;
@@ -491,7 +501,7 @@ mode_failed:
   }
 modesetting_failed:
   {
-    GST_ERROR_OBJECT (self, "Failed to set mode: %s", strerror (errno));
+    GST_ERROR_OBJECT (self, "Failed to set mode: %s", g_strerror (errno));
     goto bail;
   }
 }
@@ -560,12 +570,18 @@ ensure_allowed_caps (GstKMSSink * self, drmModeConnector * conn,
     out_caps = gst_caps_merge (out_caps, gst_caps_simplify (tmp_caps));
   }
 
+  if (gst_caps_is_empty (out_caps)) {
+    GST_DEBUG_OBJECT (self, "allowed caps is empty");
+    gst_caps_unref (out_caps);
+    return FALSE;
+  }
+
   self->allowed_caps = gst_caps_simplify (out_caps);
 
   GST_DEBUG_OBJECT (self, "allowed caps = %" GST_PTR_FORMAT,
       self->allowed_caps);
 
-  return (self->allowed_caps && !gst_caps_is_empty (self->allowed_caps));
+  return TRUE;
 }
 
 static gboolean
@@ -833,7 +849,7 @@ open_failed:
   {
     GST_ELEMENT_ERROR (self, RESOURCE, OPEN_READ_WRITE,
         ("Could not open DRM module %s", GST_STR_NULL (self->devname)),
-        ("reason: %s (%d)", strerror (errno), errno));
+        ("reason: %s (%d)", g_strerror (errno), errno));
     return FALSE;
   }
 
@@ -841,7 +857,7 @@ resources_failed:
   {
     GST_ELEMENT_ERROR (self, RESOURCE, SETTINGS,
         ("drmModeGetResources failed"),
-        ("reason: %s (%d)", strerror (errno), errno));
+        ("reason: %s (%d)", g_strerror (errno), errno));
     goto bail;
   }
 
@@ -870,7 +886,7 @@ plane_resources_failed:
   {
     GST_ELEMENT_ERROR (self, RESOURCE, SETTINGS,
         ("drmModeGetPlaneResources failed"),
-        ("reason: %s (%d)", strerror (errno), errno));
+        ("reason: %s (%d)", g_strerror (errno), errno));
     goto bail;
   }
 
@@ -1009,14 +1025,6 @@ gst_kms_sink_get_caps (GstBaseSink * bsink, GstCaps * filter)
   return out_caps;
 }
 
-static void
-ensure_kms_allocator (GstKMSSink * self)
-{
-  if (self->allocator)
-    return;
-  self->allocator = gst_kms_allocator_new (self->fd);
-}
-
 static GstBufferPool *
 gst_kms_sink_create_pool (GstKMSSink * self, GstCaps * caps, gsize size,
     gint min)
@@ -1055,7 +1063,8 @@ config_failed:
 }
 
 static gboolean
-gst_kms_sink_calculate_display_ratio (GstKMSSink * self, GstVideoInfo * vinfo)
+gst_kms_sink_calculate_display_ratio (GstKMSSink * self, GstVideoInfo * vinfo,
+    gint * scaled_width, gint * scaled_height)
 {
   guint dar_n, dar_d;
   guint video_width, video_height;
@@ -1071,8 +1080,8 @@ gst_kms_sink_calculate_display_ratio (GstKMSSink * self, GstVideoInfo * vinfo)
     gst_video_calculate_device_ratio (self->hdisplay, self->vdisplay,
         self->mm_width, self->mm_height, &dpy_par_n, &dpy_par_d);
   } else {
-    GST_VIDEO_SINK_WIDTH (self) = video_width;
-    GST_VIDEO_SINK_HEIGHT (self) = video_height;
+    *scaled_width = video_width;
+    *scaled_height = video_height;
     goto out;
   }
 
@@ -1091,24 +1100,23 @@ gst_kms_sink_calculate_display_ratio (GstKMSSink * self, GstVideoInfo * vinfo)
   /* check hd / dar_d is an integer scale factor, and scale wd with the PAR */
   if (video_height % dar_d == 0) {
     GST_DEBUG_OBJECT (self, "keeping video height");
-    GST_VIDEO_SINK_WIDTH (self) = (guint)
+    *scaled_width = (guint)
         gst_util_uint64_scale_int (video_height, dar_n, dar_d);
-    GST_VIDEO_SINK_HEIGHT (self) = video_height;
+    *scaled_height = video_height;
   } else if (video_width % dar_n == 0) {
     GST_DEBUG_OBJECT (self, "keeping video width");
-    GST_VIDEO_SINK_WIDTH (self) = video_width;
-    GST_VIDEO_SINK_HEIGHT (self) = (guint)
+    *scaled_width = video_width;
+    *scaled_height = (guint)
         gst_util_uint64_scale_int (video_width, dar_d, dar_n);
   } else {
     GST_DEBUG_OBJECT (self, "approximating while keeping video height");
-    GST_VIDEO_SINK_WIDTH (self) = (guint)
+    *scaled_width = (guint)
         gst_util_uint64_scale_int (video_height, dar_n, dar_d);
-    GST_VIDEO_SINK_HEIGHT (self) = video_height;
+    *scaled_height = video_height;
   }
 
 out:
-  GST_DEBUG_OBJECT (self, "scaling to %dx%d", GST_VIDEO_SINK_WIDTH (self),
-      GST_VIDEO_SINK_HEIGHT (self));
+  GST_DEBUG_OBJECT (self, "scaling to %dx%d", *scaled_width, *scaled_height);
 
   return TRUE;
 }
@@ -1118,44 +1126,30 @@ gst_kms_sink_set_caps (GstBaseSink * bsink, GstCaps * caps)
 {
   GstKMSSink *self;
   GstVideoInfo vinfo;
-  GstBufferPool *newpool, *oldpool;
 
   self = GST_KMS_SINK (bsink);
 
-  /* We are going to change the internal buffer pool, which means it will no
-   * longer be compatbile with the last_buffer size. Drain now, as we won't be
-   * able to do that later on. */
-  gst_kms_sink_drain (self);
-
   if (!gst_video_info_from_caps (&vinfo, caps))
     goto invalid_format;
+  self->vinfo = vinfo;
 
-  if (!gst_kms_sink_calculate_display_ratio (self, &vinfo))
+  if (!gst_kms_sink_calculate_display_ratio (self, &vinfo,
+          &GST_VIDEO_SINK_WIDTH (self), &GST_VIDEO_SINK_HEIGHT (self)))
     goto no_disp_ratio;
 
   if (GST_VIDEO_SINK_WIDTH (self) <= 0 || GST_VIDEO_SINK_HEIGHT (self) <= 0)
     goto invalid_size;
 
-  /* create a new pool for the new configuration */
-  newpool = gst_kms_sink_create_pool (self, caps, GST_VIDEO_INFO_SIZE (&vinfo),
-      2);
-  if (!newpool)
-    goto no_pool;
-
-  /* we don't activate the internal pool yet as it may not be needed */
-  oldpool = self->pool;
-  self->pool = newpool;
-
-  if (oldpool) {
-    gst_buffer_pool_set_active (oldpool, FALSE);
-    gst_object_unref (oldpool);
+  /* discard dumb buffer pool */
+  if (self->pool) {
+    gst_buffer_pool_set_active (self->pool, FALSE);
+    gst_object_unref (self->pool);
+    self->pool = NULL;
   }
 
   if (self->modesetting_enabled && !configure_mode_setting (self, &vinfo))
     goto modesetting_failed;
 
-  self->vinfo = vinfo;
-
   GST_OBJECT_LOCK (self);
   if (self->reconfigure) {
     self->reconfigure = FALSE;
@@ -1187,11 +1181,6 @@ no_disp_ratio:
         ("Error calculating the output display ratio of the video."));
     return FALSE;
   }
-no_pool:
-  {
-    /* Already warned in create_pool */
-    return FALSE;
-  }
 
 modesetting_failed:
   {
@@ -1214,6 +1203,8 @@ gst_kms_sink_propose_allocation (GstBaseSink * bsink, GstQuery * query)
 
   self = GST_KMS_SINK (bsink);
 
+  GST_DEBUG_OBJECT (self, "propose allocation");
+
   gst_query_parse_allocation (query, &caps, &need_pool);
   if (!caps)
     goto no_caps;
@@ -1324,20 +1315,20 @@ gst_kms_sink_sync (GstKMSSink * self)
   /* ERRORS */
 vblank_failed:
   {
-    GST_WARNING_OBJECT (self, "drmWaitVBlank failed: %s (%d)", strerror (-ret),
-        ret);
+    GST_WARNING_OBJECT (self, "drmWaitVBlank failed: %s (%d)",
+        g_strerror (errno), errno);
     return FALSE;
   }
 pageflip_failed:
   {
     GST_WARNING_OBJECT (self, "drmModePageFlip failed: %s (%d)",
-        strerror (-ret), ret);
+        g_strerror (errno), errno);
     return FALSE;
   }
 event_failed:
   {
-    GST_ERROR_OBJECT (self, "drmHandleEvent failed: %s (%d)", strerror (-ret),
-        ret);
+    GST_ERROR_OBJECT (self, "drmHandleEvent failed: %s (%d)",
+        g_strerror (errno), errno);
     return FALSE;
   }
 }
@@ -1404,6 +1395,8 @@ gst_kms_sink_import_dmabuf (GstKMSSink * self, GstBuffer * inbuf,
       return FALSE;
   }
 
+  ensure_kms_allocator (self);
+
   kmsmem = (GstKMSMemory *) gst_kms_allocator_get_cached (mems[0]);
   if (kmsmem) {
     GST_LOG_OBJECT (self, "found KMS mem %p in DMABuf mem %p with fb id = %d",
@@ -1436,25 +1429,69 @@ wrap_mem:
   return TRUE;
 }
 
+static gboolean
+ensure_internal_pool (GstKMSSink * self, GstVideoInfo * in_vinfo,
+    GstBuffer * inbuf)
+{
+  GstBufferPool *pool;
+  GstVideoInfo vinfo = *in_vinfo;
+  GstVideoMeta *vmeta;
+  GstCaps *caps;
+
+  if (self->pool)
+    return TRUE;
+
+  /* When cropping, the caps matches the cropped rectangle width/height, but
+   * we can retrieve the padded width/height from the VideoMeta (which is kept
+   * intact when adding crop meta */
+  if ((vmeta = gst_buffer_get_video_meta (inbuf))) {
+    vinfo.width = vmeta->width;
+    vinfo.height = vmeta->height;
+  }
+
+  caps = gst_video_info_to_caps (&vinfo);
+  pool = gst_kms_sink_create_pool (self, caps, gst_buffer_get_size (inbuf), 2);
+  gst_caps_unref (caps);
+
+  if (!pool)
+    return FALSE;
+
+  if (!gst_buffer_pool_set_active (pool, TRUE))
+    goto activate_pool_failed;
+
+  self->pool = pool;
+  return TRUE;
+
+activate_pool_failed:
+  {
+    GST_ELEMENT_ERROR (self, STREAM, FAILED, ("failed to activate buffer pool"),
+        ("failed to activate buffer pool"));
+    gst_object_unref (pool);
+    return FALSE;
+  }
+
+}
+
 static GstBuffer *
-gst_kms_sink_copy_to_dumb_buffer (GstKMSSink * self, GstBuffer * inbuf)
+gst_kms_sink_copy_to_dumb_buffer (GstKMSSink * self, GstVideoInfo * vinfo,
+    GstBuffer * inbuf)
 {
   GstFlowReturn ret;
   GstVideoFrame inframe, outframe;
   gboolean success;
   GstBuffer *buf = NULL;
 
-  if (!gst_buffer_pool_set_active (self->pool, TRUE))
-    goto activate_pool_failed;
+  if (!ensure_internal_pool (self, vinfo, inbuf))
+    goto bail;
 
   ret = gst_buffer_pool_acquire_buffer (self->pool, &buf, NULL);
   if (ret != GST_FLOW_OK)
     goto create_buffer_failed;
 
-  if (!gst_video_frame_map (&inframe, &self->vinfo, inbuf, GST_MAP_READ))
+  if (!gst_video_frame_map (&inframe, vinfo, inbuf, GST_MAP_READ))
     goto error_map_src_buffer;
 
-  if (!gst_video_frame_map (&outframe, &self->vinfo, buf, GST_MAP_WRITE))
+  if (!gst_video_frame_map (&outframe, vinfo, buf, GST_MAP_WRITE))
     goto error_map_dst_buffer;
 
   success = gst_video_frame_copy (&outframe, &inframe);
@@ -1473,12 +1510,6 @@ bail:
   }
 
   /* ERRORS */
-activate_pool_failed:
-  {
-    GST_ELEMENT_ERROR (self, STREAM, FAILED, ("failed to activate buffer pool"),
-        ("failed to activate buffer pool"));
-    return NULL;
-  }
 create_buffer_failed:
   {
     GST_ELEMENT_ERROR (self, STREAM, FAILED, ("allocation failed"),
@@ -1519,7 +1550,7 @@ gst_kms_sink_get_input_buffer (GstKMSSink * self, GstBuffer * inbuf)
     goto done;
 
   GST_CAT_INFO_OBJECT (CAT_PERFORMANCE, self, "frame copy");
-  buf = gst_kms_sink_copy_to_dumb_buffer (self, inbuf);
+  buf = gst_kms_sink_copy_to_dumb_buffer (self, &self->vinfo, inbuf);
 
 done:
   /* Copy all the non-memory related metas, this way CropMeta will be
@@ -1537,8 +1568,10 @@ gst_kms_sink_show_frame (GstVideoSink * vsink, GstBuffer * buf)
   GstBuffer *buffer = NULL;
   guint32 fb_id;
   GstKMSSink *self;
+  GstVideoInfo *vinfo;
   GstVideoCropMeta *crop;
   GstVideoRectangle src = { 0, };
+  gint video_width, video_height;
   GstVideoRectangle dst = { 0, };
   GstVideoRectangle result;
   GstFlowReturn res;
@@ -1547,10 +1580,17 @@ gst_kms_sink_show_frame (GstVideoSink * vsink, GstBuffer * buf)
 
   res = GST_FLOW_ERROR;
 
-  if (buf)
+  if (buf) {
     buffer = gst_kms_sink_get_input_buffer (self, buf);
-  else if (self->last_buffer)
+    vinfo = &self->vinfo;
+    video_width = src.w = GST_VIDEO_SINK_WIDTH (self);
+    video_height = src.h = GST_VIDEO_SINK_HEIGHT (self);
+  } else if (self->last_buffer) {
     buffer = gst_buffer_ref (self->last_buffer);
+    vinfo = &self->last_vinfo;
+    video_width = src.w = self->last_width;
+    video_height = src.h = self->last_height;
+  }
 
   /* Make sure buf is not used accidentally */
   buf = NULL;
@@ -1570,20 +1610,19 @@ gst_kms_sink_show_frame (GstVideoSink * vsink, GstBuffer * buf)
   }
 
   if ((crop = gst_buffer_get_video_crop_meta (buffer))) {
-    GstVideoInfo vinfo = self->vinfo;
-    vinfo.width = crop->width;
-    vinfo.height = crop->height;
+    GstVideoInfo cropped_vinfo = *vinfo;
 
-    if (!gst_kms_sink_calculate_display_ratio (self, &vinfo))
+    cropped_vinfo.width = crop->width;
+    cropped_vinfo.height = crop->height;
+
+    if (!gst_kms_sink_calculate_display_ratio (self, &cropped_vinfo, &src.w,
+            &src.h))
       goto no_disp_ratio;
 
     src.x = crop->x;
     src.y = crop->y;
   }
 
-  src.w = GST_VIDEO_SINK_WIDTH (self);
-  src.h = GST_VIDEO_SINK_HEIGHT (self);
-
   dst.w = self->render_rect.w;
   dst.h = self->render_rect.h;
 
@@ -1597,8 +1636,8 @@ retry_set_plane:
     src.w = crop->width;
     src.h = crop->height;
   } else {
-    src.w = GST_VIDEO_INFO_WIDTH (&self->vinfo);
-    src.h = GST_VIDEO_INFO_HEIGHT (&self->vinfo);
+    src.w = video_width;
+    src.h = video_height;
   }
 
   /* handle out of screen case */
@@ -1642,8 +1681,13 @@ sync_frame:
     goto bail;
   }
 
-  if (buffer != self->last_buffer)
+  /* Save the rendered buffer and its metadata in case a redraw is needed */
+  if (buffer != self->last_buffer) {
     gst_buffer_replace (&self->last_buffer, buffer);
+    self->last_width = GST_VIDEO_SINK_WIDTH (self);
+    self->last_height = GST_VIDEO_SINK_HEIGHT (self);
+    self->last_vinfo = self->vinfo;
+  }
   g_clear_pointer (&self->tmp_kmsmem, gst_memory_unref);
 
   GST_OBJECT_UNLOCK (self);
@@ -1667,7 +1711,7 @@ set_plane_failed:
         result.w, result.h, src.x, src.y, src.w, src.h, dst.x, dst.y, dst.w,
         dst.h);
     GST_ELEMENT_ERROR (self, RESOURCE, FAILED,
-        (NULL), ("drmModeSetPlane failed: %s (%d)", strerror (-ret), ret));
+        (NULL), ("drmModeSetPlane failed: %s (%d)", g_strerror (errno), errno));
     goto bail;
   }
 no_disp_ratio:
@@ -1684,8 +1728,6 @@ gst_kms_sink_drain (GstKMSSink * self)
 {
   GstParentBufferMeta *parent_meta;
 
-  GST_DEBUG_OBJECT (self, "draining");
-
   if (!self->last_buffer)
     return;
 
@@ -1693,11 +1735,24 @@ gst_kms_sink_drain (GstKMSSink * self)
    * In this case, the last_buffer will have a GstParentBufferMeta set. */
   parent_meta = gst_buffer_get_parent_buffer_meta (self->last_buffer);
   if (parent_meta) {
-    GstBuffer *dumb_buf;
-    dumb_buf = gst_kms_sink_copy_to_dumb_buffer (self, parent_meta->buffer);
+    GstBuffer *dumb_buf, *last_buf;
+
+    /* If this was imported from our dumb buffer pool we can safely skip the
+     * drain */
+    if (parent_meta->buffer->pool &&
+        GST_IS_KMS_BUFFER_POOL (parent_meta->buffer->pool))
+      return;
+
+    GST_DEBUG_OBJECT (self, "draining");
+
+    dumb_buf = gst_kms_sink_copy_to_dumb_buffer (self, &self->last_vinfo,
+        parent_meta->buffer);
+    last_buf = self->last_buffer;
+    self->last_buffer = dumb_buf;
+
     gst_kms_allocator_clear_cache (self->allocator);
-    gst_kms_sink_show_frame (GST_VIDEO_SINK (self), dumb_buf);
-    gst_buffer_unref (dumb_buf);
+    gst_kms_sink_show_frame (GST_VIDEO_SINK (self), NULL);
+    gst_buffer_unref (last_buf);
   }
 }
 
@@ -2007,7 +2062,7 @@ gst_kms_sink_class_init (GstKMSSinkClass * klass)
    */
   g_properties[PROP_CONNECTOR_PROPS] =
       g_param_spec_boxed ("connector-properties", "Connector Properties",
-      "Additionnal properties for the connector",
+      "Additional properties for the connector",
       GST_TYPE_STRUCTURE, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
   /**
@@ -2020,7 +2075,7 @@ gst_kms_sink_class_init (GstKMSSinkClass * klass)
    */
   g_properties[PROP_PLANE_PROPS] =
       g_param_spec_boxed ("plane-properties", "Connector Plane",
-      "Additionnal properties for the plane",
+      "Additional properties for the plane",
       GST_TYPE_STRUCTURE, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
   g_object_class_install_properties (gobject_class, PROP_N, g_properties);
@@ -2031,11 +2086,7 @@ gst_kms_sink_class_init (GstKMSSinkClass * klass)
 static gboolean
 plugin_init (GstPlugin * plugin)
 {
-  if (!gst_element_register (plugin, GST_PLUGIN_NAME, GST_RANK_SECONDARY,
-          GST_TYPE_KMS_SINK))
-    return FALSE;
-
-  return TRUE;
+  return GST_ELEMENT_REGISTER (kmssink, plugin);
 }
 
 GST_PLUGIN_DEFINE (GST_VERSION_MAJOR, GST_VERSION_MINOR, kms,
