@@ -22,6 +22,7 @@
 #include "config.h"
 #endif
 
+#include <gst/base/base.h>
 #include <gst/video/video.h>
 #include "gstmfsourcereader.h"
 #include <string.h>
@@ -30,6 +31,7 @@
 #include <vector>
 #include <algorithm>
 
+/* *INDENT-OFF* */
 using namespace Microsoft::WRL;
 
 G_BEGIN_DECLS
@@ -38,6 +40,7 @@ GST_DEBUG_CATEGORY_EXTERN (gst_mf_source_object_debug);
 #define GST_CAT_DEFAULT gst_mf_source_object_debug
 
 G_END_DECLS
+/* *INDENT-ON* */
 
 typedef struct _GstMFStreamMediaType
 {
@@ -71,8 +74,9 @@ struct _GstMFSourceReader
   GMainLoop *loop;
 
   /* protected by lock */
-  GQueue *queue;
+  GstQueueArray *queue;
 
+  IMFActivate *activate;
   IMFMediaSource *source;
   IMFSourceReader *reader;
 
@@ -81,23 +85,33 @@ struct _GstMFSourceReader
   GstMFStreamMediaType *cur_type;
   GstVideoInfo info;
 
+  gboolean top_down_image;
+
   gboolean flushing;
 };
 
+typedef struct _GstMFSourceReaderSample
+{
+  IMFSample *sample;
+  GstClockTime clock_time;
+} GstMFSourceReaderSample;
+
 static void gst_mf_source_reader_constructed (GObject * object);
 static void gst_mf_source_reader_finalize (GObject * object);
 
 static gboolean gst_mf_source_reader_start (GstMFSourceObject * object);
-static gboolean gst_mf_source_reader_stop  (GstMFSourceObject * object);
+static gboolean gst_mf_source_reader_stop (GstMFSourceObject * object);
 static GstFlowReturn gst_mf_source_reader_fill (GstMFSourceObject * object,
     GstBuffer * buffer);
 static GstFlowReturn gst_mf_source_reader_create (GstMFSourceObject * object,
     GstBuffer ** buffer);
 static gboolean gst_mf_source_reader_unlock (GstMFSourceObject * object);
 static gboolean gst_mf_source_reader_unlock_stop (GstMFSourceObject * object);
-static GstCaps * gst_mf_source_reader_get_caps (GstMFSourceObject * object);
+static GstCaps *gst_mf_source_reader_get_caps (GstMFSourceObject * object);
 static gboolean gst_mf_source_reader_set_caps (GstMFSourceObject * object,
     GstCaps * caps);
+static void
+gst_mf_source_reader_sample_clear (GstMFSourceReaderSample * reader_sample);
 
 static gboolean gst_mf_source_reader_open (GstMFSourceReader * object,
     IMFActivate * activate);
@@ -134,7 +148,10 @@ gst_mf_source_reader_class_init (GstMFSourceReaderClass * klass)
 static void
 gst_mf_source_reader_init (GstMFSourceReader * self)
 {
-  self->queue = g_queue_new ();
+  self->queue =
+      gst_queue_array_new_for_struct (sizeof (GstMFSourceReaderSample), 2);
+  gst_queue_array_set_clear_func (self->queue,
+      (GDestroyNotify) gst_mf_source_reader_sample_clear);
   g_mutex_init (&self->lock);
   g_cond_init (&self->cond);
 }
@@ -163,14 +180,21 @@ gst_mf_enum_media_type_from_source_reader (IMFSourceReader * source_reader,
   gint i, j;
   HRESULT hr;
   GList *list = NULL;
-  std::vector<std::string> unhandled_caps;
+  std::vector < std::string > unhandled_caps;
 
   g_return_val_if_fail (source_reader != NULL, FALSE);
   g_return_val_if_fail (media_types != NULL, FALSE);
 
-  for (i = 0;; i++) {
+  {
+    /* Retrive only the first video stream. non-first video stream might be
+     * photo stream which doesn't seem to be working propertly in this implementation.
+     *
+     * Note: Chromium seems to be using the first video stream
+     * https://github.com/chromium/chromium/blob/ccd149af47315e4c6f2fc45d55be1b271f39062c/media/capture/video/win/video_capture_device_factory_win.cc
+     */
+    i = MF_SOURCE_READER_FIRST_VIDEO_STREAM;
     for (j = 0;; j++) {
-      ComPtr<IMFMediaType> media_type;
+      ComPtr < IMFMediaType > media_type;
 
       hr = source_reader->GetNativeMediaType (i, j, &media_type);
 
@@ -190,10 +214,10 @@ gst_mf_enum_media_type_from_source_reader (IMFSourceReader * source_reader,
         name = gst_structure_get_name (s);
         if (name != "video/x-raw" && name != "image/jpeg") {
           auto it =
-              std::find(unhandled_caps.begin(), unhandled_caps.end(), name);
-          if (it == unhandled_caps.end()) {
-            GST_FIXME ("Skip not supported format %s", name.c_str());
-            unhandled_caps.push_back(name);
+              std::find (unhandled_caps.begin (), unhandled_caps.end (), name);
+          if (it == unhandled_caps.end ()) {
+            GST_FIXME ("Skip not supported format %s", name.c_str ());
+            unhandled_caps.push_back (name);
           }
           gst_caps_unref (caps);
           continue;
@@ -227,7 +251,7 @@ done:
   list = g_list_reverse (list);
   *media_types = list;
 
-  return ! !list;
+  return !!list;
 }
 
 static void
@@ -260,11 +284,11 @@ gst_mf_source_reader_open (GstMFSourceReader * self, IMFActivate * activate)
 {
   GList *iter;
   HRESULT hr;
-  ComPtr<IMFSourceReader> reader;
-  ComPtr<IMFMediaSource> source;
-  ComPtr<IMFAttributes> attr;
+  ComPtr < IMFSourceReader > reader;
+  ComPtr < IMFMediaSource > source;
+  ComPtr < IMFAttributes > attr;
 
-  hr = activate->ActivateObject (IID_IMFMediaSource, (void **) &source);
+  hr = activate->ActivateObject (IID_PPV_ARGS (&source));
   if (!gst_mf_result (hr))
     return FALSE;
 
@@ -288,6 +312,8 @@ gst_mf_source_reader_open (GstMFSourceReader * self, IMFActivate * activate)
     return FALSE;
   }
 
+  self->activate = activate;
+  activate->AddRef ();
   self->source = source.Detach ();
   self->reader = reader.Detach ();
 
@@ -313,6 +339,12 @@ gst_mf_source_reader_close (GstMFSourceReader * self)
 {
   gst_clear_caps (&self->supported_caps);
 
+  if (self->activate) {
+    self->activate->ShutdownObject ();
+    self->activate->Release ();
+    self->activate = NULL;
+  }
+
   if (self->media_types) {
     g_list_free_full (self->media_types,
         (GDestroyNotify) gst_mf_stream_media_type_free);
@@ -343,7 +375,7 @@ gst_mf_source_reader_finalize (GObject * object)
   g_main_loop_unref (self->loop);
   g_main_context_unref (self->context);
 
-  g_queue_free (self->queue);
+  gst_queue_array_free (self->queue);
   gst_clear_caps (&self->supported_caps);
   g_mutex_clear (&self->lock);
   g_cond_clear (&self->cond);
@@ -364,12 +396,38 @@ gst_mf_source_reader_start (GstMFSourceObject * object)
   }
 
   type = self->cur_type;
+  self->top_down_image = TRUE;
 
   if (GST_VIDEO_INFO_FORMAT (&self->info) != GST_VIDEO_FORMAT_ENCODED) {
-    hr = type->media_type->SetUINT32 (MF_MT_DEFAULT_STRIDE,
-        GST_VIDEO_INFO_PLANE_STRIDE (&self->info, 0));
-    if (!gst_mf_result (hr))
-      return FALSE;
+    UINT32 stride;
+    INT32 actual_stride = GST_VIDEO_INFO_PLANE_STRIDE (&self->info, 0);
+
+    /* This MF_MT_DEFAULT_STRIDE uses UINT32 type but actual value is
+     * INT32, which can be negative in case of RGB image, and negative means
+     * its stored as bottom-up manner */
+    hr = type->media_type->GetUINT32 (MF_MT_DEFAULT_STRIDE, &stride);
+    if (gst_mf_result (hr)) {
+      actual_stride = (INT32) stride;
+      if (actual_stride < 0) {
+        if (!GST_VIDEO_INFO_IS_RGB (&self->info)) {
+          GST_ERROR_OBJECT (self,
+              "Bottom-up image is allowed only for RGB format");
+          return FALSE;
+        }
+
+        GST_DEBUG_OBJECT (self,
+            "Detected bottom-up image, stride %d", actual_stride);
+
+        self->top_down_image = FALSE;
+      }
+    } else {
+      /* If MF_MT_DEFAULT_STRIDE attribute is not specified, we can use our
+       * value */
+      type->media_type->SetUINT32 (MF_MT_DEFAULT_STRIDE,
+          (UINT32) actual_stride);
+    }
+    gst_mf_update_video_info_with_stride (&self->info,
+        std::abs (actual_stride));
   }
 
   hr = self->reader->SetStreamSelection (type->stream_index, TRUE);
@@ -384,15 +442,23 @@ gst_mf_source_reader_start (GstMFSourceObject * object)
   return TRUE;
 }
 
+static GstMFSourceReaderSample *
+gst_mf_source_reader_sample_new (IMFSample * sample, GstClockTime timestamp)
+{
+  GstMFSourceReaderSample *reader_sample = g_new0 (GstMFSourceReaderSample, 1);
+
+  reader_sample->sample = sample;
+  reader_sample->clock_time = timestamp;
+
+  return reader_sample;
+}
+
 static gboolean
 gst_mf_source_reader_stop (GstMFSourceObject * object)
 {
   GstMFSourceReader *self = GST_MF_SOURCE_READER (object);
 
-  while (!g_queue_is_empty (self->queue)) {
-    IMFMediaBuffer *buffer = (IMFMediaBuffer *) g_queue_pop_head (self->queue);
-    buffer->Release ();
-  }
+  gst_queue_array_clear (self->queue);
 
   return TRUE;
 }
@@ -401,47 +467,55 @@ static GstFlowReturn
 gst_mf_source_reader_read_sample (GstMFSourceReader * self)
 {
   HRESULT hr;
-  DWORD count = 0, i;
   DWORD stream_flags = 0;
   GstMFStreamMediaType *type = self->cur_type;
-  ComPtr<IMFSample> sample;
+  IMFSample *sample = nullptr;
+  GstMFSourceReaderSample reader_sample;
 
   hr = self->reader->ReadSample (type->stream_index, 0, NULL, &stream_flags,
-    NULL, &sample);
+      NULL, &sample);
 
-  if (!gst_mf_result (hr))
+  if (!gst_mf_result (hr)) {
+    GST_ERROR_OBJECT (self, "Failed to read sample");
     return GST_FLOW_ERROR;
-
-  if ((stream_flags & MF_SOURCE_READERF_ERROR) == MF_SOURCE_READERF_ERROR)
-    return GST_FLOW_ERROR;
-
-  if (!sample)
-    return GST_FLOW_OK;
-
-  hr = sample->GetBufferCount (&count);
-  if (!gst_mf_result (hr) || !count)
-    return GST_FLOW_OK;
-
-  for (i = 0; i < count; i++) {
-    IMFMediaBuffer *buffer = NULL;
-
-    hr = sample->GetBufferByIndex (i, &buffer);
-    if (!gst_mf_result (hr) || !buffer)
-      continue;
-
-    g_queue_push_tail (self->queue, buffer);
   }
 
+  if ((stream_flags & MF_SOURCE_READERF_ERROR) == MF_SOURCE_READERF_ERROR) {
+    GST_ERROR_OBJECT (self, "Error while reading sample, sample flags 0x%x",
+        stream_flags);
+    return GST_FLOW_ERROR;
+  }
+
+  if (!sample) {
+    GST_WARNING_OBJECT (self, "Empty sample");
+    return GST_FLOW_OK;
+  }
+
+  reader_sample.sample = sample;
+  reader_sample.clock_time =
+      gst_mf_source_object_get_running_time (GST_MF_SOURCE_OBJECT (self));
+
+  gst_queue_array_push_tail_struct (self->queue, &reader_sample);
+
   return GST_FLOW_OK;
 }
 
 static GstFlowReturn
 gst_mf_source_reader_get_media_buffer (GstMFSourceReader * self,
-    IMFMediaBuffer ** media_buffer)
+    IMFMediaBuffer ** buffer, GstClockTime * timestamp, GstClockTime * duration)
 {
   GstFlowReturn ret = GST_FLOW_OK;
+  IMFSample *sample = nullptr;
+  HRESULT hr;
+  DWORD count = 0;
+  LONGLONG mf_timestamp;
+  GstMFSourceReaderSample *reader_sample = nullptr;
 
-  while (g_queue_is_empty (self->queue)) {
+  *buffer = nullptr;
+  *timestamp = GST_CLOCK_TIME_NONE;
+  *duration = GST_CLOCK_TIME_NONE;
+
+  while (gst_queue_array_is_empty (self->queue)) {
     ret = gst_mf_source_reader_read_sample (self);
     if (ret != GST_FLOW_OK)
       return ret;
@@ -454,7 +528,37 @@ gst_mf_source_reader_get_media_buffer (GstMFSourceReader * self,
     g_mutex_unlock (&self->lock);
   }
 
-  *media_buffer = (IMFMediaBuffer *) g_queue_pop_head (self->queue);
+  reader_sample =
+      (GstMFSourceReaderSample *) gst_queue_array_pop_head_struct (self->queue);
+  sample = reader_sample->sample;
+  g_assert (sample);
+
+  hr = sample->GetBufferCount (&count);
+  if (!gst_mf_result (hr) || count == 0) {
+    GST_WARNING_OBJECT (self, "Empty IMFSample, read again");
+    goto done;
+  }
+
+  /* XXX: read the first buffer and ignore the others for now */
+  hr = sample->GetBufferByIndex (0, buffer);
+  if (!gst_mf_result (hr)) {
+    GST_WARNING_OBJECT (self, "Couldn't get IMFMediaBuffer from sample");
+    goto done;
+  }
+
+  hr = sample->GetSampleDuration (&mf_timestamp);
+  if (!gst_mf_result (hr)) {
+    GST_WARNING_OBJECT (self, "Couldn't get sample duration");
+    *duration = GST_CLOCK_TIME_NONE;
+  } else {
+    /* Media Foundation uses 100 nano seconds unit */
+    *duration = mf_timestamp * 100;
+  }
+
+  *timestamp = reader_sample->clock_time;
+
+done:
+  gst_mf_source_reader_sample_clear (reader_sample);
 
   return GST_FLOW_OK;
 }
@@ -464,13 +568,19 @@ gst_mf_source_reader_fill (GstMFSourceObject * object, GstBuffer * buffer)
 {
   GstMFSourceReader *self = GST_MF_SOURCE_READER (object);
   GstFlowReturn ret = GST_FLOW_OK;
-  ComPtr<IMFMediaBuffer> media_buffer;
+  ComPtr < IMFMediaBuffer > media_buffer;
   GstVideoFrame frame;
   BYTE *data;
   gint i, j;
   HRESULT hr;
+  GstClockTime timestamp = GST_CLOCK_TIME_NONE;
+  GstClockTime duration = GST_CLOCK_TIME_NONE;
+
+  do {
+    ret = gst_mf_source_reader_get_media_buffer (self,
+        media_buffer.ReleaseAndGetAddressOf (), &timestamp, &duration);
+  } while (ret == GST_FLOW_OK && !media_buffer);
 
-  ret = gst_mf_source_reader_get_media_buffer (self, &media_buffer);
   if (ret != GST_FLOW_OK)
     return ret;
 
@@ -486,29 +596,57 @@ gst_mf_source_reader_fill (GstMFSourceObject * object, GstBuffer * buffer)
     return GST_FLOW_ERROR;
   }
 
-  for (i = 0; i < GST_VIDEO_INFO_N_PLANES (&self->info); i++) {
+  if (!self->top_down_image) {
     guint8 *src, *dst;
     gint src_stride, dst_stride;
-    gint width;
+    gint width, height;
 
-    src = data + GST_VIDEO_INFO_PLANE_OFFSET (&self->info, i);
-    dst = (guint8 *) GST_VIDEO_FRAME_PLANE_DATA (&frame, i);
+    /* must be single plane RGB */
+    width = GST_VIDEO_INFO_COMP_WIDTH (&self->info, 0)
+        * GST_VIDEO_INFO_COMP_PSTRIDE (&self->info, 0);
+    height = GST_VIDEO_INFO_HEIGHT (&self->info);
 
-    src_stride = GST_VIDEO_INFO_PLANE_STRIDE (&self->info, i);
-    dst_stride = GST_VIDEO_FRAME_PLANE_STRIDE (&frame, i);
-    width = GST_VIDEO_INFO_COMP_WIDTH (&self->info, i)
-        * GST_VIDEO_INFO_COMP_PSTRIDE (&self->info, i);
+    src_stride = GST_VIDEO_INFO_PLANE_STRIDE (&self->info, 0);
+    dst_stride = GST_VIDEO_FRAME_PLANE_STRIDE (&frame, 0);
 
-    for (j = 0; j < GST_VIDEO_INFO_COMP_HEIGHT (&self->info, i); j++) {
+    /* This is bottom up image, should copy lines in reverse order */
+    src = data + src_stride * (height - 1);
+    dst = (guint8 *) GST_VIDEO_FRAME_PLANE_DATA (&frame, 0);
+
+    for (j = 0; j < height; j++) {
       memcpy (dst, src, width);
-      src += src_stride;
+      src -= src_stride;
       dst += dst_stride;
     }
+  } else {
+    for (i = 0; i < GST_VIDEO_INFO_N_PLANES (&self->info); i++) {
+      guint8 *src, *dst;
+      gint src_stride, dst_stride;
+      gint width;
+
+      src = data + GST_VIDEO_INFO_PLANE_OFFSET (&self->info, i);
+      dst = (guint8 *) GST_VIDEO_FRAME_PLANE_DATA (&frame, i);
+
+      src_stride = GST_VIDEO_INFO_PLANE_STRIDE (&self->info, i);
+      dst_stride = GST_VIDEO_FRAME_PLANE_STRIDE (&frame, i);
+      width = GST_VIDEO_INFO_COMP_WIDTH (&self->info, i)
+          * GST_VIDEO_INFO_COMP_PSTRIDE (&self->info, i);
+
+      for (j = 0; j < GST_VIDEO_INFO_COMP_HEIGHT (&self->info, i); j++) {
+        memcpy (dst, src, width);
+        src += src_stride;
+        dst += dst_stride;
+      }
+    }
   }
 
   gst_video_frame_unmap (&frame);
   media_buffer->Unlock ();
 
+  GST_BUFFER_PTS (buffer) = timestamp;
+  GST_BUFFER_DTS (buffer) = GST_CLOCK_TIME_NONE;
+  GST_BUFFER_DURATION (buffer) = duration;
+
   return GST_FLOW_OK;
 }
 
@@ -517,14 +655,20 @@ gst_mf_source_reader_create (GstMFSourceObject * object, GstBuffer ** buffer)
 {
   GstMFSourceReader *self = GST_MF_SOURCE_READER (object);
   GstFlowReturn ret = GST_FLOW_OK;
-  ComPtr<IMFMediaBuffer> media_buffer;
+  ComPtr < IMFMediaBuffer > media_buffer;
   HRESULT hr;
   BYTE *data;
   DWORD len = 0;
   GstBuffer *buf;
   GstMapInfo info;
+  GstClockTime timestamp = GST_CLOCK_TIME_NONE;
+  GstClockTime duration = GST_CLOCK_TIME_NONE;
+
+  do {
+    ret = gst_mf_source_reader_get_media_buffer (self,
+        media_buffer.ReleaseAndGetAddressOf (), &timestamp, &duration);
+  } while (ret == GST_FLOW_OK && !media_buffer);
 
-  ret = gst_mf_source_reader_get_media_buffer (self, &media_buffer);
   if (ret != GST_FLOW_OK)
     return ret;
 
@@ -547,6 +691,11 @@ gst_mf_source_reader_create (GstMFSourceObject * object, GstBuffer ** buffer)
 
   media_buffer->Unlock ();
 
+  GST_BUFFER_PTS (buf) = timestamp;
+  /* Set DTS since this is compressed format */
+  GST_BUFFER_DTS (buf) = timestamp;
+  GST_BUFFER_DURATION (buf) = duration;
+
   *buffer = buf;
 
   return GST_FLOW_OK;
@@ -596,7 +745,7 @@ gst_mf_source_reader_set_caps (GstMFSourceObject * object, GstCaps * caps)
 
   for (iter = self->media_types; iter; iter = g_list_next (iter)) {
     GstMFStreamMediaType *minfo = (GstMFStreamMediaType *) iter->data;
-    if (gst_caps_is_subset (minfo->caps, caps)) {
+    if (gst_caps_can_intersect (minfo->caps, caps)) {
       best_type = minfo;
       break;
     }
@@ -725,7 +874,7 @@ gst_mf_source_enum_device_activate (GstMFSourceReader * self,
 {
   HRESULT hr;
   GList *ret = NULL;
-  ComPtr<IMFAttributes> attr;
+  ComPtr < IMFAttributes > attr;
   IMFActivate **devices = NULL;
   UINT32 i, count = 0;
 
@@ -759,9 +908,9 @@ gst_mf_source_enum_device_activate (GstMFSourceReader * self,
 
     switch (source_type) {
       case GST_MF_SOURCE_TYPE_VIDEO:
-        hr = activate->GetAllocatedString (
-            MF_DEVSOURCE_ATTRIBUTE_SOURCE_TYPE_VIDCAP_SYMBOLIC_LINK,
-            &name, &name_len);
+        hr = activate->GetAllocatedString
+            (MF_DEVSOURCE_ATTRIBUTE_SOURCE_TYPE_VIDCAP_SYMBOLIC_LINK, &name,
+            &name_len);
         break;
       default:
         g_assert_not_reached ();
@@ -795,7 +944,7 @@ done:
 
   *device_sources = ret;
 
-  return ! !ret;
+  return !!ret;
 }
 
 static void
@@ -811,6 +960,19 @@ gst_mf_device_activate_free (GstMFDeviceActivate * activate)
   g_free (activate);
 }
 
+static void
+gst_mf_source_reader_sample_clear (GstMFSourceReaderSample * reader_sample)
+{
+  if (!reader_sample)
+    return;
+
+  if (reader_sample->sample)
+    reader_sample->sample->Release ();
+
+  reader_sample->sample = nullptr;
+  reader_sample->clock_time = GST_CLOCK_TIME_NONE;
+}
+
 GstMFSourceObject *
 gst_mf_source_reader_new (GstMFSourceType type, gint device_index,
     const gchar * device_name, const gchar * device_path)
@@ -833,4 +995,4 @@ gst_mf_source_reader_new (GstMFSourceType type, gint device_index,
   }
 
   return self;
-}
\ No newline at end of file
+}
