@@ -74,6 +74,10 @@ static const struct map gst_msdk_video_format_to_mfx_map[] = {
   GST_VIDEO_INFO_TO_MFX_MAP (Y212_LE, YUV422, Y216),
   /* Y416 is used for 12bit 4:4:4:4 format in MSDK */
   GST_VIDEO_INFO_TO_MFX_MAP (Y412_LE, YUV444, Y416),
+#endif
+#if (MFX_VERSION >=2004)
+  GST_VIDEO_INFO_TO_MFX_MAP (RGBP, YUV444, RGBP),
+  GST_VIDEO_INFO_TO_MFX_MAP (BGRP, YUV444, BGRP),
 #endif
   {0, 0, 0}
 };
@@ -123,10 +127,12 @@ msdk_status_to_string (mfxStatus status)
       return "device operation failure";
     case MFX_ERR_MORE_BITSTREAM:
       return "expect more bitstream buffers at output";
+#if (MFX_VERSION < 2000)
     case MFX_ERR_INCOMPATIBLE_AUDIO_PARAM:
       return "incompatible audio parameters";
     case MFX_ERR_INVALID_AUDIO_PARAM:
       return "invalid audio parameters";
+#endif
       /* warnings >0 */
     case MFX_WRN_IN_EXECUTION:
       return "the previous asynchronous operation is in execution";
@@ -144,8 +150,10 @@ msdk_status_to_string (mfxStatus status)
       return "the value is out of valid range";
     case MFX_WRN_FILTER_SKIPPED:
       return "one of requested filters has been skipped";
+#if (MFX_VERSION < 2000)
     case MFX_WRN_INCOMPATIBLE_AUDIO_PARAM:
       return "incompatible audio parameters";
+#endif
     default:
       break;
   }
@@ -170,8 +178,149 @@ msdk_get_platform_codename (mfxSession session)
   return codename;
 }
 
+#if (MFX_VERSION >= 2000)
+
+mfxStatus
+msdk_init_msdk_session (mfxIMPL impl, mfxVersion * pver,
+    MsdkSession * msdk_session)
+{
+  mfxStatus sts = MFX_ERR_NONE;
+  mfxLoader loader = NULL;
+  mfxSession session = NULL;
+  uint32_t impl_idx = 0;
+  mfxConfig cfg;
+  mfxVariant impl_value;
+
+  loader = msdk_session->loader;
+
+  if (!loader) {
+    loader = MFXLoad ();
+
+    GST_INFO ("Use the Intel oneVPL SDK to create MFX session");
+
+    if (!loader) {
+      GST_WARNING ("Failed to create a MFX loader");
+      return MFX_ERR_UNKNOWN;
+    }
+
+    /* Create configurations for implementation */
+    cfg = MFXCreateConfig (loader);
+
+    if (!cfg) {
+      GST_ERROR ("Failed to create a MFX configuration");
+      MFXUnload (loader);
+      return MFX_ERR_UNKNOWN;
+    }
+
+    impl_value.Type = MFX_VARIANT_TYPE_U32;
+    impl_value.Data.U32 =
+        (impl ==
+        MFX_IMPL_SOFTWARE) ? MFX_IMPL_TYPE_SOFTWARE : MFX_IMPL_TYPE_HARDWARE;
+    sts =
+        MFXSetConfigFilterProperty (cfg,
+        (const mfxU8 *) "mfxImplDescription.Impl", impl_value);
+
+    if (sts != MFX_ERR_NONE) {
+      GST_ERROR ("Failed to add an additional MFX configuration (%s)",
+          msdk_status_to_string (sts));
+      MFXUnload (loader);
+      return sts;
+    }
+
+    impl_value.Type = MFX_VARIANT_TYPE_U32;
+    impl_value.Data.U32 = pver->Version;
+    sts =
+        MFXSetConfigFilterProperty (cfg,
+        (const mfxU8 *) "mfxImplDescription.ApiVersion.Version", impl_value);
+
+    if (sts != MFX_ERR_NONE) {
+      GST_ERROR ("Failed to add an additional MFX configuration (%s)",
+          msdk_status_to_string (sts));
+      MFXUnload (loader);
+      return sts;
+    }
+  }
+
+  while (1) {
+    /* Enumerate all implementations */
+    mfxImplDescription *impl_desc;
+
+    sts = MFXEnumImplementations (loader, impl_idx,
+        MFX_IMPLCAPS_IMPLDESCSTRUCTURE, (mfxHDL *) & impl_desc);
+
+    /* Failed to find an available implementation */
+    if (sts == MFX_ERR_NOT_FOUND)
+      break;
+    else if (sts != MFX_ERR_NONE) {
+      impl_idx++;
+      continue;
+    }
+
+    sts = MFXCreateSession (loader, impl_idx, &session);
+    MFXDispReleaseImplDescription (loader, impl_desc);
+
+    if (sts == MFX_ERR_NONE)
+      break;
+
+    impl_idx++;
+  }
+
+  if (sts != MFX_ERR_NONE) {
+    GST_ERROR ("Failed to create a MFX session (%s)",
+        msdk_status_to_string (sts));
+
+    if (!msdk_session->loader)
+      MFXUnload (loader);
+
+    return sts;
+  }
+
+  msdk_session->session = session;
+  msdk_session->loader = loader;
+
+  return MFX_ERR_NONE;
+}
+
+#else
+
+mfxStatus
+msdk_init_msdk_session (mfxIMPL impl, mfxVersion * pver,
+    MsdkSession * msdk_session)
+{
+  mfxStatus status;
+  mfxSession session = NULL;
+  mfxInitParam init_par = { impl, *pver };
+
+  GST_INFO ("Use the " MFX_API_SDK " to create MFX session");
+
+#if (MFX_VERSION >= 1025)
+  init_par.GPUCopy = 1;
+#endif
+
+  status = MFXInitEx (init_par, &session);
+
+  if (status != MFX_ERR_NONE) {
+    GST_WARNING ("Failed to initialize a MFX session (%s)",
+        msdk_status_to_string (status));
+    return status;
+  }
+
+  msdk_session->session = session;
+  msdk_session->loader = NULL;
+
+  return MFX_ERR_NONE;
+}
+
 void
-msdk_close_session (mfxSession session)
+MFXUnload (mfxLoader loader)
+{
+  g_assert (loader == NULL);
+}
+
+#endif
+
+void
+msdk_close_mfx_session (mfxSession session)
 {
   mfxStatus status;
 
@@ -183,7 +332,14 @@ msdk_close_session (mfxSession session)
     GST_ERROR ("Close failed (%s)", msdk_status_to_string (status));
 }
 
-mfxSession
+void
+msdk_close_session (MsdkSession * msdk_session)
+{
+  msdk_close_mfx_session (msdk_session->session);
+  MFXUnload (msdk_session->loader);
+}
+
+MsdkSession
 msdk_open_session (mfxIMPL impl)
 {
   mfxSession session = NULL;
@@ -191,19 +347,21 @@ msdk_open_session (mfxIMPL impl)
   };
   mfxIMPL implementation;
   mfxStatus status;
-  mfxU16 codename;
+  MsdkSession msdk_session;
 
   static const gchar *implementation_names[] = {
     "AUTO", "SOFTWARE", "HARDWARE", "AUTO_ANY", "HARDWARE_ANY", "HARDWARE2",
     "HARDWARE3", "HARDWARE4", "RUNTIME"
   };
 
-  status = MFXInit (impl, &version, &session);
-  if (status != MFX_ERR_NONE) {
-    GST_ERROR ("Intel Media SDK not available (%s)",
-        msdk_status_to_string (status));
-    goto failed;
-  }
+  msdk_session.session = NULL;
+  msdk_session.loader = NULL;
+  status = msdk_init_msdk_session (impl, &version, &msdk_session);
+
+  if (status != MFX_ERR_NONE)
+    return msdk_session;
+  else
+    session = msdk_session.session;
 
   status = MFXQueryIMPL (session, &implementation);
   if (status != MFX_ERR_NONE) {
@@ -218,33 +376,30 @@ msdk_open_session (mfxIMPL impl)
     goto failed;
   }
 
-  codename = msdk_get_platform_codename (session);
-
-  if (codename != MFX_PLATFORM_UNKNOWN)
-    GST_INFO ("Detected MFX platform with device code %d", codename);
-  else
-    GST_WARNING ("Unknown MFX platform");
-
   GST_INFO ("MFX implementation: 0x%04x (%s)", implementation,
       implementation_names[MFX_IMPL_BASETYPE (implementation)]);
   GST_INFO ("MFX version: %d.%d", version.Major, version.Minor);
 
-  return session;
+  return msdk_session;
 
 failed:
-  msdk_close_session (session);
-  return NULL;
+  msdk_close_session (&msdk_session);
+  msdk_session.session = NULL;
+  msdk_session.loader = NULL;
+  return msdk_session;
 }
 
 gboolean
 msdk_is_available (void)
 {
-  mfxSession session = msdk_open_session (MFX_IMPL_HARDWARE_ANY);
-  if (!session) {
+  /* Make sure we can create GstMsdkContext instance (the job type is not used actually) */
+  GstMsdkContext *msdk_context = gst_msdk_context_new (1, GST_MSDK_JOB_DECODER);
+
+  if (!msdk_context) {
     return FALSE;
   }
 
-  msdk_close_session (session);
+  gst_object_unref (msdk_context);
   return TRUE;
 }
 
@@ -489,6 +644,7 @@ gboolean
 gst_msdk_load_plugin (mfxSession session, const mfxPluginUID * uid,
     mfxU32 version, const gchar * plugin)
 {
+#if (MFX_VERSION < 2000)
   mfxStatus status;
 
   status = MFXVideoUSER_Load (session, uid, version);
@@ -503,6 +659,7 @@ gst_msdk_load_plugin (mfxSession session, const mfxPluginUID * uid,
     GST_WARNING ("Media SDK Plugin for %s load warning: %s", plugin,
         msdk_status_to_string (status));
   }
+#endif
 
   return TRUE;
 }
